#
# command.py - dtv2ser high level commands
#
# Written by
#  Christian Vogelgsang <chris@vogelgsang.org>
#
# This file is part of dtv2ser.
# See README for copyright notice.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
#  02111-1307  USA.
#

from dtv2ser.status   import *
from dtv2ser.cmdline  import *
from dtv2ser.sercon   import *
from dtv2ser.transfer import *
from dtv2ser.autotype import *
from dtv2ser.joystream import *
from dtv2ser.screencode import *

class Command:
  """The high level commands of dtv2ser."""

  # client version: 0.3
  client_major = 0
  client_minor = 3

  def __init__(self,con):
    self.cmdline   = CmdLine(con)
    self.transfer  = Transfer(con)
    self.autoType  = AutoType()
    self.joyStream = JoyStream()
  
  # ----- version -----------------------------------------------------------

  def get_server_version(self):
    """Return version of dtv2ser firmware.
    Returns (status,major,minor)
    """
    result = self.cmdline.do_command('v')
    if result != STATUS_OK:
      return (result,0,0)
    (result,version) = self.cmdline.get_word()
    major = version >> 8
    minor = version & 0xff
    return (result,major,minor)

  def get_client_version(self):
    """Return version of client"""
    return (self.client_major,self.client_minor)

  def is_compatible_version(self,server_major,server_minor):
    """Check if host and server versions are compatible.
    Return True or False
    """
    return self.client_major == server_major and \
           self.client_minor == server_minor

  # ----- dtv client commands -----------------------------------------------
  
  def read_memory(self,rom,start,length,callback=lambda x:True):
    """Read a memory block from the DTV.
    Return (result,data,client_rx_rate,server_rx_rate).
    """
    cmd = "r%02x%06x%06x" % (rom,start,length)
    self.transfer.begin_rx_rates()
    (result,data) = self.transfer.do_receive_command(cmd,start,length,callback=callback);
    stat          = self.transfer.get_rx_rates(length)
    return (result,data,stat)

  def write_memory(self,rom,start,data,callback=lambda x:True):
    """Write a memory block to the DTV
    Return (result,client_tx_rate,server_tx_rate).
    """
    cmd = "w%02x%06x%06x" % (rom,start,len(data))
    self.transfer.begin_tx_rates()
    result    = self.transfer.do_send_command(cmd,start,data,callback=callback)
    stat      = self.transfer.get_tx_rates(len(data))
    return (result,stat)

  def write_boot_memory(self,start,data,callback=lambda x:True):
    """Write a boot memory block to the DTV
    Return (result,client_tx_rate,server_tx_rate).
    """
    self.transfer.begin_tx_rates()
    result    = self.transfer.do_send_boot(start,data,callback=callback)
    stat      = self.transfer.get_tx_rates(len(data))
    return (result,stat)

  def dtv_reset(self):
    """Reset the DTV and enter dtvtrans if dtvmon is available.
    Return result.
    """
    result = self.cmdline.do_command("x")
    if result != STATUS_OK:
      return result
    (result,val) = self.cmdline.get_byte()
    return result
  
  def go_memory(self,addr):
    """Go/jump to memory location.
    Return result.
    """
    cmd = "g%04x" % addr
    result = self.cmdline.do_command(cmd)
    if result != STATUS_OK:
      return result
    (result,val) = self.cmdline.get_byte()
    return result

  # ----- joystick ----------------------------------------------------------
  
  def joy_wiggle(self,duration,delay,callback=lambda x:True):
    """Wiggle joystick to enter command prompt in intro"""
    stream = self.joyStream.generate_wiggle_stream(duration,delay)
    print "\t%d seconds of wiggling with %d ms delay" % (duration,delay*10)
    print "\tsending joy stream with %d bytes" % len(stream)
    return self.transfer.do_joy_stream(stream,callback)

  def joy_rawkey(self,delta,delay,callback=lambda x:True):
    """Press a single key on the virtual DTV keyboard by specifying the
       delta move"""
    stream = self.joyStream.generate_delta_move_stream(delta,delay)
    print "\tpressing raw key at",delta,"with",delay*10,"ms delay"
    print "\tsending joy stream with %d bytes" % len(stream)
    return self.transfer.do_joy_stream(stream,callback)

  def joy_stream(self,string,
                 joy_delay=0,
                 verbose=False,
                 test_mode=False,
                 callback=lambda x:True):
    """Execute a joy stream given in string notation"""
    if joy_delay == 0:
      joy_delay = 6
    js_seq,total = self.joyStream.parse_string_stream_seq(string,joy_delay)
    print "\tjoy string: converted string with %d bytes and %d ms start delay" % (len(string),joy_delay*10)
    print "\tjoy stream: sending stream with %d segments and %d bytes" % (len(js_seq),total)
    if verbose:
      print "\tdump:",self.joyStream.dump_stream_seq(js_seq)
    if test_mode:
      return STATUS_OK,0
    else:
      return self.transfer.do_joy_stream_seq(js_seq,callback)

  def joy_type(self,string,
               screen_code=False,
               type_delay=0,
               joy_delay=0,
               start_key='',
               verbose=False,
               test_mode=False,
               callback=lambda x:True):
    """Type a string on the virtual DTV keyboard"""

    # set auto type defaults
    status=self.autoType.setup(start_key,type_delay,joy_delay)
    if status != STATUS_OK:
      return status
    
    # conver string from auto type
    if screen_code:
      sc = ScreenCode(verbose,self.autoType,self.joyStream) 
      at_string = sc.convert_binary_to_type_string(string)
      print "\tscreen code: converting %d bytes to %d auto type string bytes" % (len(string),len(at_string))
      if verbose:
        print "\tdump:",at_string
      string = at_string
    
    at_seq=self.autoType.convert_string_to_autotype_seq(string)
    print "\tauto type:  typing %d joy string bytes with %d sequence entries" % (len(string),len(at_seq))
    js_seq,total_cmds = self.joyStream.convert_autotype_seq(at_seq)
    print "\tjoy stream: sending stream with %d segment(s) and %d total bytes" % (len(js_seq),total_cmds)
    if verbose:
      print "\test. duration:",self.joyStream.estimate_stream_seq_duration(js_seq)
      print "\tjoy str. dump:",self.joyStream.dump_stream_seq(js_seq)
    if test_mode:
      return STATUS_OK,0
    else:
      return self.transfer.do_joy_stream_seq(js_seq,callback)
  
  # ----- diagnose commands -------------------------------------------------

  def diagnose_read_memory_only_dtv(self,rom,start,length,callback=lambda x:True):
    """Read a memory block from the DTV but do not receive the data via serial.
    Return (result,stat).
    """
    cmd = "ddr%02x%06x%06x" % (rom,start,length)
    self.transfer.begin_rx_rates()
    result = self.transfer.do_diagnose_command_only_dtv(cmd,length,False,callback=callback)
    stat = self.transfer.get_rx_rates(length)
    return (result,stat)

  def diagnose_write_memory_only_dtv(self,rom,start,length,callback=lambda x:True,pattern=0):
    """Write a memory block to the DTV but do not send the data via serial.
    Return (result,stat).
    """
    if pattern != 0:
      result = self.transfer.set_diagnose_pattern(pattern)
      if result != STATUS_OK:
        return result

    cmd = "ddw%02x%06x%06x" % (rom,start,length)
    self.transfer.begin_tx_rates()
    result = self.transfer.do_diagnose_command_only_dtv(cmd,length,True,callback=callback)
    stat = self.transfer.get_tx_rates(length)
    return (result,stat)

  def diagnose_verify_memory_only_dtv(self,rom,start,length,callback=lambda x:True,pattern=0):
    """Read and verify a memory block fromo the DTV but do not send the data via serial.
    Return (result,stat).
    """
    if pattern != 0:
      result = self.transfer.set_diagnose_pattern(pattern)
      if result != STATUS_OK:
        return result

    cmd = "ddv%02x%06x%06x" % (rom,start,length)
    self.transfer.begin_rx_rates()
    result = self.transfer.do_diagnose_command_only_dtv(cmd,length,False,callback=callback)
    stat = self.transfer.get_rx_rates(length)
    return (result,stat)

  def diagnose_read_memory_only_client(self,length,callback=lambda x:True,pattern=0):
    """Read a memory block from the dtv2ser device but do not transfer from DTV.
    Return (result,stat).
    """
    if pattern != 0:
      result = self.transfer.set_diagnose_pattern(pattern)
      if result != STATUS_OK:
        return result

    cmd = "dsr00000000%06x" % length
    self.transfer.begin_rx_rates()
    result = self.transfer.do_diagnose_receive_command_only_client(cmd,0,length,callback=callback)
    stat = self.transfer.get_rx_rates(length)
    return (result,stat)
    
  def diagnose_write_memory_only_client(self,length,callback=lambda x:True,pattern=0):
    """Write a memory block to the dtv2ser device but do not transfer to the DTV.
    Return (result,stat).
    """
    if pattern != 0:
      result = self.transfer.set_diagnose_pattern(pattern)
      if result != STATUS_OK:
        return result

    cmd = "dsw00000000%06x" % length
    self.transfer.begin_tx_rates()
    result = self.transfer.do_diagnose_send_command_only_client(cmd,0,length,callback=callback)
    stat = self.transfer.get_tx_rates(length)
    return (result,stat)

  def diagnose_do_command(self,cmd):
    """Issue a command directly.
    Return result
    """
    return self.cmdline.do_command(cmd)

  # ----- high level commands -----------------------------------------------

  def read_basic_size(self):
    """Read the basic start and end from DTV
    
    Return (result,basis_start,basic_end)
    """
    (result,data,stat) = self.read_memory(0,0x2b,4)
    if result != 0:
      return (result,0,0)
    basic_start = ord(data[0]) + 256 * ord(data[1])
    basic_end   = ord(data[2]) + 256 * ord(data[3])
    if basic_end < basic_start:
      return (CLIENT_ERROR_INVALID_BASIC_RANGE,0,0)
    return (result,basic_start,basic_end)
  
